import { useEffect, useState } from "react";
import useMediaQuery from "@mui/material/useMediaQuery";
import { useTheme } from "@mui/material/styles";
import Button from "@mui/material/Button";
import DialogTitle from "@mui/material/DialogTitle";
import DialogContent from "@mui/material/DialogContent";
import DialogActions from "@mui/material/DialogActions";
import Dialog from "@mui/material/Dialog";
import IconButton from "@mui/material/IconButton";
import Toolbar from "@mui/material/Toolbar";
import Typography from "@mui/material/Typography";
import Stack from "@mui/material/Stack";
import DownloadIcon from "@mui/icons-material/Download";
import { getBackupUrl, getServerBackups } from "../../util/queries";

export function useBackupDialog(isOpen = false) {
  const [open, setOpen] = useState(isOpen);
  const dialogToggle = () => setOpen(!open);
  return [open, dialogToggle];
}

export default function BackupDialog(props) {
  const { serverId, open, dialogToggle } = props;
  const theme = useTheme();
  const fullScreen = useMediaQuery(theme.breakpoints.down("md"));
  const [backups, setBackups] = useState([]);

  function refreshUpdateList() {
    getServerBackups(serverId).then(setBackups);
  }
  useEffect(() => {
    if (!serverId) return;
    refreshUpdateList();
  }, [serverId, open]);

  function normalizeLastModified(lastModified) {
    const d = new Date(Date.parse(lastModified));
    return `${d.getFullYear()}-${d.getMonth()}-${d.getDate()} ${d.getHours()}:${d.getMinutes()}`;
  }

  const downloadBackup = (backup) =>
    async function openBackupLink() {
      const { url } = await getBackupUrl(serverId, backup.path);
      window.open(url, "_blank").focus();
    };

  const normalizedSize = (size) => `${(size / Math.pow(1024, 3)).toFixed(2)}GB`;

  return (
    <Dialog
      fullWidth
      maxWidth="lg"
      open={open}
      fullScreen={fullScreen}
      PaperProps={!fullScreen ? { sx: { height: "60%" } } : undefined}
    >
      <Toolbar sx={{ display: { md: "none" } }} />
      <DialogTitle>Backups</DialogTitle>
      <DialogContent sx={{ height: "100%" }}>
        {backups.map((backup, i) => (
          <Stack key={i} sx={{ width: "100%" }} direction="row">
            <Typography variant="subtitle2" sx={{ m: "auto 0", width: "40%" }}>
              {backup.name}
            </Typography>
            <Typography variant="subtitle2" sx={{ m: "auto 0", width: "20%" }}>
              {normalizeLastModified(backup.lastModified)}
            </Typography>
            <Typography variant="subtitle2" sx={{ m: "auto 0", width: "40%" }}>
              {normalizedSize(backup.size)}
            </Typography>

            <IconButton
              sx={{ marginLeft: "auto" }}
              onClick={downloadBackup(backup)}
            >
              <DownloadIcon />
            </IconButton>
          </Stack>
        ))}
      </DialogContent>
      <DialogActions>
        <Button autoFocus onClick={dialogToggle}>
          Close
        </Button>
      </DialogActions>
    </Dialog>
  );
}
